# -*- coding: utf-8 -*-
import os
from os.path import dirname
from cStringIO import StringIO
from abc import ABCMeta, abstractmethod
from tempfile import NamedTemporaryFile

from lxml import etree
from contextlib import contextmanager
from jinja2 import Environment, FileSystemLoader
from jinja2.exceptions import UndefinedError

from .tools import tools


@contextmanager
def change_path():
    old_path = os.getcwd()
    try:
        yield
        os.chdir(old_path)
    finally:
        os.chdir(old_path)


class Struct(object):
    def __init__(self, adict):
        """Convert a dictionary to a class

        @param :adict Dictionary
        """
        self.__dict__.update(adict)
        for k, v in adict.items():
            if isinstance(v, dict):
                self.__dict__[k] = Struct(v)
            if isinstance(v, list):
                # import pdb; pdb.set_trace()
                self.__dict__[k] = [Struct(x) for x in v]


class BaseDocument:
    """An XML document following any format given an XSD file.
    Due to avoid duplication of work we will delegate the
    error management of attributes to the xsd, then the
    `validate` method will make the job of return the correct
    error, due to the standard managed on the invoice.

    The template itself must comply with an specific xsd,
    this is needed to simply pass a dictionary of terms used in
    the template convert them to
    attributes of this Document object using whatever attributes
    comes from that xsd

    Then due to the template itself has all the structure of
    attributes necessaries to comply with the xsd, theoretically
    the xsd should return the logical error which
    we are not complying on such template, see cfdv32.xml
    template to see how you should assembly a new version
    of this template, then set it to the template_fname attribute
    and guala your dict will be magically validated
    and converted to an XML file.

    **Why not assembly with simple lxml?**

    Because it is more readable and configurable,
    it is always more simple
    inherit a class and set an attribute than overwrite
    hundreds of methods when it is a big xml.
    """

    @abstractmethod
    def __init__(self, dict_document, debug_mode=False, cache=1000):
        """Convert a dictionary invoice to a Class with a
        based xsd and xslt element to be signed.

        :param dict dict_document: Dictionary with all entries
            you will need in your template.
        :param bool debug_mode: If debugging or not.
        :param int cache: Time in seconds the url given
            files will be cached on tmp folder.
        """
        self.ups = False
        self.debug_mode = debug_mode
        self.schema_url = None
        self.document = ''
        self.document_path = None
        self.xslt_path = None
        self.xslt_document = None
        self.set_schema_fname()
        self.set_schema(self.schema_fname)
        self.__dict__.update(dict_document)
        for k, v in dict_document.items():
            if isinstance(v, dict):
                self.__dict__[k] = Struct(v)
            if isinstance(v, list):
                # import pdb; pdb.set_trace()
                self.__dict__[k] = [Struct(x) for x in v]

        self.set_xml()
        self.set_xslt_fname()
        self.document_orginal = self.set_original()
        self.template_fname = ''
        self.schema_fname = self.template_fname.replace('.xml', '.xsd')
        self.xslt_fname = self.template_fname.replace('.xml', '.xslt')
        self.templates = os.path.join(dirname(__file__), 'templates')

    __metaclass__ = ABCMeta

    def set_original(self):
        if self.document_path and self.xslt_fname:
            return tools.get_original(self.document_path, self.xslt_fname)

    def set_schema_fname(self):
        """The same than template but with .xsd on templates folder."""
        self.schema_fname = self.template_fname.replace('.xml', '.xsd')

    def set_xslt_fname(self):
        """The same than template but with .xslt on templates
        folder this in case you want to use it locally."""
        self.set_xslt()

    def guess_autoescape(self, template_name):
        """Given a template Name I will gues using its
        extension if we should autoscape or not.
        Default autoscaped extensions: ('html', 'xhtml', 'htm', 'xml')
        """
        if template_name is None or '.' not in template_name:
            return False
        ext = template_name.rsplit('.', 1)[1]
        return ext in ('html', 'xhtml', 'htm', 'xml')

    @abstractmethod
    def set_schema(self, schema_fname):
        test_xml = os.path.join(self.templates, schema_fname)
        with open(test_xml, 'r') as element:
            schema = element.read()
        return schema

    @abstractmethod
    def set_xslt(self):
        if self.xslt_fname and tools.is_url(self.xslt_fname):
            xslt_path = tools.cache_it(self.xslt_fname)
        with open(xslt_path, 'r') as element:
            xslt = element.read()
            self.xslt_document = xslt
            # In case of caching,
            # the xslt_path will be from cached and not from local
            self.xslt_fname = xslt_path

    @abstractmethod
    def set_template(self, template_fname):
        self.templates = os.path.join(dirname(__file__), 'templates')
        env = Environment(loader=FileSystemLoader(self.templates),
                          extensions=['jinja2.ext.autoescape'],
                          autoescape=self.guess_autoescape)
        return env.get_template(template_fname)

    def validate(self, schema_str, xml_valid):
        """Compare the valid information on an xml from  given schema.

        :param str schema_str: content string from schema file.
        :param str xml_valid: content string from xml file.
        :returns: If it is Valid or Not.
        :rtype: bool
        """
        # TODO: be able to get doc for error given an xsd.
        # Changed path to allow have xsd that are imported by others xsd in the
        # same library, and not call to SAT page each time that is generated
        # a new XML.
        with change_path():
            path = os.path.join(
                os.path.dirname(os.path.realpath(__file__)), 'templates')
            os.chdir(path)
            schema_root = etree.parse(StringIO(schema_str))
            schema = etree.XMLSchema(schema_root)
        try:
            tree = etree.parse(StringIO(xml_valid.encode('UTF-8')))
            schema.assertValid(tree)
        except etree.DocumentInvalid as ups:
            self.ups = ups
        finally:
            if self.ups:
                self.valid = False
            else:
                self.valid = True
            return self.valid

    def set_xml(self):
        """Set document xml just rendered already
        validated against xsd to be signed.

        :params boolean debug_mode: Either if you want
            the rendered template to be saved either it
        is valid or not with the given schema.
        :returns boolean: Either was valid or not the generated document.
        """
        cached = StringIO()
        document = u''
        try:
            document = self.template.render(inv=self)
        except UndefinedError as ups:
            self.ups = ups

        # TODO: Here should be called the cleanup 'Just before the validation'.
        valid = self.validate(self.schema, document)
        self.document = document
        if valid:
            document = etree.XML(document)
            self.document = etree.tostring(document,
                                           pretty_print=True,
                                           xml_declaration=True,
                                           encoding='utf-8')
            # TODO: When Document Generated, this this should not fail either.
            # Caching just when valid then.
            cached.write(self.document is not None and self.document or u'')
            cached.seek(0)
            self.document_path = cached

    def get_element_from_clark(self, element):
        """**Helper method:** Given a Clark's Notation
        `{url:schema}Element` element, return the
        valid xpath on your xsd file, frequently
        it is not necesary overwrite this method but
        different xsd from different source
        can have different logic which I do not know now,
        then simply take this as an example and set the
        correct xpath conversion in your project.

        :param str element: Element string following the Clark's Notation"""
        element = element.split('}')[-1]
        xpath_path = \
            '//xs:element[@name="{element}"]' + \
            '/xs:annotation/xs:documentation'.format(element=element)
        return xpath_path

    def get_documentation(self, element, namespace=None, schema_str=None):
        """**Helper method:** should return an schema specific documentation
        given an element parsing or getting the `Clark's Notation`_
        `{url:schema}Element` from the message error on validate method.

        :param str element: Element string following the Clark's Notation
        :param dict namespace: Element string following the Clark's Notation

        :returns: The documentation text if exists
        :rtype: unicode

        .. _`Clark's Notation`: http://effbot.org/zone/element-namespaces.htm
        """
        if namespace is None:
            namespace = {'xs': 'http://www.w3.org/2001/XMLSchema'}
        schema_root = etree.parse(StringIO(self.schema))
        document = schema_root.xpath(self.get_element_from_clark(element),
                                     namespaces=namespace)
        return document and document[0].text or ''



# -*- coding: utf-8 -*-
from __future__ import print_function
import logging
import boto3
import re
import time
import urllib2

from contextlib import closing

from boto3.s3.transfer import S3Transfer
from botocore.client import ClientError
from tempfile import NamedTemporaryFile
from lxml import etree
from os.path import isfile
from functools import wraps
from urlparse import urlparse

try:
    from functools import lru_cache
except ImportError:
    from backports.functools_lru_cache import lru_cache

_logger = logging.getLogger(__name__)


def retry(exception_to_check, tries=4, delay=3, back_off=2, logger=None):  # pragma: no cover  # noqa
    """Retry calling the decorated function using an exponential backoff.

    http://www.saltycrane.com/blog/2009/11/trying-out-retry-decorator-python/
    original from: http://wiki.python.org/moin/PythonDecoratorLibrary#Retry

    :param exception_to_check: the exception to check. may be a tuple of
        exceptions to check
    :type exception_to_check: Exception or tuple
    :param tries: number of times to try (not retry) before giving up
    :type tries: int
    :param delay: initial delay between retries in seconds
    :type delay: int
    :param back_off: back_off multiplier e.g. value of 2 will double the
        delay each retry
    :type back_off: int
    :param logger: logger to use. If None, print
    :type logger: logging.Logger instance
    """

    def deco_retry(f):

        @wraps(f)
        def f_retry(*args, **kwargs):
            mtries, mdelay = tries, delay
            while mtries > 1:
                try:
                    return f(*args, **kwargs)
                except exception_to_check as e:
                    msg = "%s, Retrying in %d seconds..." % (str(e),
                                                             mdelay)
                    if logger:
                        logger.warning(msg)
                    else:
                        print(msg)
                    time.sleep(mdelay)
                    mtries -= 1
                    mdelay *= back_off
            return f(*args, **kwargs)

        return f_retry  # true decorator

    return deco_retry


class Tools(object):
    """Set of tools to manipulate things that generally will be common between
    several ways to manage signed files and/or external connections.

    Basically if a method is not legally required but technically necessary
    that method will be here."""

    def __init__(self, domain='s3.vauxoo.com', profile_name='dev'):
        self.cached = {}
        self.domain = domain
        self.profile_name = profile_name

    def s3_url(self, url):
        return url.replace(urlparse(url).netloc, self.domain)

    def send_s3_xsd(self, url_xsd):  # pragma: no cover
        """This method will not be re-run always, only locally and when xsd
        are regenerated, read the test_008_force_s3_creation on test folder
        """

        if self.check_s3(self.domain, urlparse(url_xsd).path[1:]):
            return url_xsd

        response = urllib2.urlopen(url_xsd)
        content = response.read()
        cached = NamedTemporaryFile(delete=False)
        named = cached.name

        # Find all urls in the main xslt file.

        urls = re.findall(r'href=[\'"]?([^\'" >]+)', content)

        # mapping in the main file the url's

        for original_url in urls:
            content = content.replace(
                original_url, self.s3_url(original_url))

        with cached as cache:
            cache.write(content)

        created_url = self.cache_s3(url_xsd, named)
        print('Created Url Ok!: %s' % created_url)

        # Mapping all internal url in the file to s3 cached env.
        for original_url in urls:

            # Expecting 1 level of deepest links in xsd if more, refactor this.

            response = urllib2.urlopen(original_url)
            content = response.read()

            # Find all urls in the main xslt file.

            in_urls = re.findall(r'href=[\'"]?([^\'" >]+)', content)

            # mapping in the main file the url's

            for orig_url in in_urls:
                content = content.replace(
                    orig_url, self.s3_url(orig_url))

            cached = NamedTemporaryFile(delete=False)
            with cached as cache:
                cache.write(content)
            named = cached.name

            new_url = self.cache_s3(original_url, named)
            print('Created Url Ok!: %s' % new_url)

        return created_url

    def check_s3(self, bucket, element):  # pragma: no cover
        """This method is a helper con `cache_s3`.
        Read method `cache_s3` for more information.

        :param bucket:
        :param element:
        :return:
        """
        session = boto3.Session(profile_name=self.profile_name)
        s3 = session.resource('s3')

        try:
            s3.meta.client.head_bucket(Bucket=bucket)
        except ClientError:
            # If the bucket does not exists then simply use the original
            # I silently fail returning everything as it is in the url
            return False
        try:
            # If the key does not exists do not return False, but try to
            # create a readonly user in order to not have problems into the
            # travis environment.
            s3.Object(bucket, element).load()
        except ClientError:
            return False
        else:
            return True

    def cache_s3(self, url, named):  # pragma: no cover
        """Basically this is not to deploy automatically this is to be run once
        all is properly defined to catch the xsd in your own S3 instance and
        avoid third party (like government servers) failure, trying to manage
        the cache transparently to the user.

        This method was created due to SAT is too unstable to serve the files
        related with xsd and invoices, and this probably is the same in other
        governmental institutions.

        :param url: Element path to be cached.
        :type url: str
        :param named: Local path with the file already downloaded.
        :type named: str

        :return:
        """
        # **Technical Notes:**
        #
        # Even if the tries are looking nested, this is perfect valid for
        # this case:
        # https://docs.python.org/3/glossary.html#term-eafp.
        #
        # The Coverage was excluded in order to avoid decrease it because an
        # unused tool.
        url_parsed = urlparse(url)
        if self.domain == urlparse(url).netloc:
            # If I am asking for the same domain it is probably because it
            # exists BTW
            return url

        # Get the service resource
        session = boto3.Session(profile_name=self.profile_name)
        s3 = session.resource('s3')
        client = session.client('s3')
        element = url_parsed.path[1:]

        # Get the bucket object (I am not connected yet this is setting a
        # local object in memory only)
        bucket = s3.Bucket(self.domain)

        # Dear future me there MUST be a more elegant way to do this.
        new_url = 'http://%s/%s' % (self.domain, element)

        if self.check_s3(bucket.name, element):
            return new_url

        # Look a way this code is not tested in coverage because this MUST run
        # locally due to s3 credentials.
        try:  # pragma: no cover
            # No coverage code unreachable on travis 'intentionally'
            transfer = S3Transfer(client)
            # Making the object public
            transfer.upload_file(named, bucket.name, element,
                                 extra_args={'ACL': 'public-read'})
        except ClientError as inst:  # pragma: no cover
            # No coverage code unreachable on travis 'intentionally'
            print(inst.message)
            return url
        else:  # pragma: no cover
            # No coverage code unreachable on travis 'intentionally'
            return new_url

    @staticmethod
    def is_url(element_name):
        return element_name.startswith('http')

    @staticmethod
    @lru_cache(maxsize=128)
    @retry(urllib2.URLError, tries=3, delay=3, back_off=2)
    def _cache_it(url):
        with closing(urllib2.urlopen(url)) as f_url:
            content = f_url.read()
            rep = r'href=[\'"]?([^\'" >]+)'
            # TODO: unwire this to xslt, but usefull for now
            # Mapping all external url in the file to local cached files.
            sub_urls = filter(lambda u: not isfile(u), re.findall(rep, content)
                              ) if url.endswith('xslt') else []
            for s_url in sub_urls:
                content = content.replace(s_url, Tools._cache_it(s_url).name)
            cached = NamedTemporaryFile()
            cached.write(content)
            cached.seek(0)
            return cached

    def cache_it(self, url):
        """Take an url which deliver a plain document  and convert it to a
        temporary file, this document is an xslt file expecting contains all
        xslt definitions, then the cache process is recursive.

        :param url: document origin url
        :type url: str

        :return file_path: local new absolute path
        :rtype file_path: str
        """
        # TODO: Use directly the file object instead of the name of the file
        #       with seek(0)
        cached = self._cache_it(url)
        if not isfile(cached.name):
            # If /tmp files are deleted
            self._cache_it.cache_clear()
            cached = self._cache_it(url)
        return cached.name

    @staticmethod
    def get_original(document, xslt):
        """Get the original chain given document path and xslt local path

        :param str document: local absolute path to document
        :param str xslt: local absolute path to xst file
        :return: new chain generated.
        :rtype: str
        """
        dom = etree.parse(document)  # TODO: cuando este probando -
        # fuente:
        # http://stackoverflow.com/questions/16698935/how-to-transform-an-xml-file-using-xslt-in-python
        xslt = etree.parse(xslt)
        transform = etree.XSLT(xslt)
        newdom = transform(dom)
        return newdom


tools = Tools()



#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""
test_cfdilib
----------------------------------

Tests for `cfdilib` module.
Tests for `cfdv32` module.
"""

from os.path import join, dirname, isfile
from os import environ, unlink
import unittest

from cfdilib import cfdilib, cfdv32, cfdv33
from cfdilib.tools import tools


class TestCfdilib(unittest.TestCase):

    def _get_test_file(self, file_name):
        test_data_file = join(self.test_data_dir, file_name)
        with open(test_data_file) as data_file:
            return data_file.read()

    def setUp(self):
        self.test_url_plain = 'http://www.textfiles.com/ufo/bc_grav1.txt'
        self.s3_url_plain = 'http://s3.vauxoo.com/ufo/bc_grav1.txt'
        # This is the xslt original in order to force the saving recursive in
        # amazon s3 when local.
        self.cadena = 'http://www.sat.gob.mx/sitio_internet/cfd/3/cadenaoriginal_3_2/cadenaoriginal_3_2.xslt'
        self.cadena_travis = tools.s3_url('http://s3.vauxoo.com/sitio_internet/cfd/3/cadenaoriginal_3_2/cadenaoriginal_3_2.xslt')
        self.test_data_dir = join(
            dirname(cfdilib.__file__), "..", "tests", "demo")
        self.dict_invoice_basic_32 = eval(
            self._get_test_file('basic_invoice_32.txt'))
        self.dict_invoice_basic_32_errored = eval(
            self._get_test_file('basic_invoice_32_errored.txt'))
        self.dict_invoice_basic_32_false = eval(
            self._get_test_file('basic_invoice_32_false.txt'))
        self.dict_invoice_basic_32_norfc = eval(
            self._get_test_file('basic_noRFC.txt'))
        self.dict_coa = eval(
            self._get_test_file('coa.txt'))
        self.dict_balance = eval(
            self._get_test_file('balance.txt'))
        self.dict_moves = eval(
            self._get_test_file('moves.txt'))
        self.dict_payroll = eval(
            self._get_test_file('payroll.txt'))
        self.dict_payment = eval(
            self._get_test_file('payment.txt'))
        self.dict_cfdi33 = eval(self._get_test_file('basic_invoice_33.txt'))
        self.dict_invoice_basic_33_errored = eval(
            self._get_test_file('basic_invoice_33_errored.txt'))
        self.dict_invoice_basic_33_false = eval(
            self._get_test_file('basic_invoice_33_false.txt'))
        self.real_document_xml = join(
            dirname(cfdilib.__file__), "..", "tests", "demo", 'cfdv32.xml')
        self.test_plain = join(
            dirname(cfdilib.__file__), "..", "tests", "demo", "hello.txt")

    def tearDown(self):
        pass

    def test_001_get_xsd_documentation(self):
        """Getting a documentation from a given Clark's Notated xsd element"""
        invoice = cfdv32.get_invoice(self.dict_invoice_basic_32)
        documentation = invoice.get_documentation(
                '{http://www.sat.gob.mx/cfd/3}'
                'Impuestos')
        self.assertTrue(True)  # TODO: Find the proper search element syntax

    def test_002_get_cfd_debugged(self):
        """With a given valid dict an
        invoice object is created in debug_mode"""
        invoice = cfdv32.get_invoice(self.dict_invoice_basic_32,
                                     debug_mode=True)
        self.assertFalse(bool(invoice.ups),
                         'A valid dictionary gave error debugged_mode enabled %s' % invoice.document)
        self.assertTrue(invoice.document,
                        'A valid dictionary gave error debugged_mode enabled')

    def test_002_get_coa(self):
        """With a given valid dict an
        invoice object is created in debug_mode"""
        coa = cfdv32.get_coa(self.dict_coa,
                             debug_mode=True)
        self.assertTrue(coa.document,
                        'A valid dictionary gave error with the coa')
        self.assertFalse(bool(coa.ups),
                         'A valid dictionary gave error coa %s' % coa.document)

    def test_002_get_balance(self):
        """With a given valid dict an
        invoice object is created in debug_mode"""
        balance = cfdv32.get_balance(self.dict_balance,
                                     debug_mode=True)
        self.assertFalse(bool(balance.ups),
                         'A valid dictionary gave error coa %s' % balance.document)
        self.assertTrue(balance.document,
                        'A valid dictionary gave error with the coa')

    def test_002_get_moves(self):
        """With a given valid dict an
        invoice object is created in debug_mode"""
        moves = cfdv32.get_moves(self.dict_moves,
                                 debug_mode=True)
        self.assertFalse(bool(moves.ups),
                         'A valid dictionary gave error coa %s' % moves.document)
        self.assertTrue(moves.document,
                        'A valid dictionary gave error with the coa')

    def test_003_get_cfd(self):
        """With a given valid dict an invoice object is created"""
        invoice = cfdv32.get_invoice(self.dict_invoice_basic_32)
        self.assertTrue(invoice.document,
                        'A valid dictionary gave error an error')

    def test_004_get_errored(self):
        """With a given invalid dict raise properly errors on ups object"""
        invoice = cfdv32.get_invoice(self.dict_invoice_basic_32_errored)
        self.assertTrue(bool(invoice.ups),
                        'An invalid dictionary gave a '
                        'valid output, that is wrong.')
        # Ok it failed!, then we assert if
        # the message is the one I expected for.
        self.assertTrue(invoice.ups.message.find('Emisor') > 0,
                        'The expected failed entry Emisor was erroneous. %s' % invoice.ups.message)

        invoice = cfdv32.get_invoice({})
        self.assertTrue(bool(invoice.ups),
                        'An empty dict should give me the validation')

        invoice = cfdv32.get_invoice(self.dict_invoice_basic_32_false)
        self.assertNotIn('False', invoice.ups.message,
                         'Passing a False value return a False string which is'
                         'incorrect  %s ' % invoice.ups.message)

    def test_005_get_cfd_invalid_debugged(self):
        """With a given `invalid` dict an invoice
        object is created in debug_mode"""
        invoice = cfdv32.get_invoice(
            self.dict_invoice_basic_32_errored, debug_mode=True)
        self.assertTrue(invoice.document,
                        'A invalid dictionary gave error debugged_mode '
                        'enabled gave an error.')

    def test_005_get_norfc(self):
        """With a given `invalid` dict an invoice
        object is created in debug_mode"""
        invoice = cfdv32.get_invoice(
            self.dict_invoice_basic_32_norfc, debug_mode=True)
        self.assertTrue(invoice.document,
                        'A invalid dictionary gave error debugged_mode '
                        'enabled gave an error.')
        self.assertTrue(bool(invoice.ups),
                        'Error expected and not received.')
        self.assertTrue(invoice.ups.message.find('rfc') >= 0,
                        'Expected a controlled error mentioning RFC and gotten other thing.')

    def test_006_download_file(self):
        """With a file it is downloaded and cached in a temporary file"""
        # TODO: Mock this
        downloaded = tools.cache_it(self.cadena_travis)
        content = open(downloaded).read()
        self.assertTrue(content.find('se establece que la salida') > 0,
                        'I read the content of a cached file and '
                        'the result was not correct.')

    def test_007_cache(self):
        """With a file it is downloaded and cached in a temporary file"""
        # TODO: Mock this
        downloaded = tools.cache_it(self.cadena_travis)
        hits = tools._cache_it.cache_info().hits
        new_downloaded = tools.cache_it(self.cadena_travis)
        new_hits = tools._cache_it.cache_info().hits
        self.assertEqual(new_hits - hits, 1,
                         'Cache was not cached properly')
        self.assertEqual(downloaded, new_downloaded, 'Cache different values')
        self.assertTrue(isfile(downloaded), 'Cache was not generate file')

    def test_007_clear_cache(self):
        """With a cached tempfile deleted"""
        # TODO: Mock this
        downloaded = tools.cache_it(self.cadena_travis)
        unlink(downloaded)
        tools.cache_it(self.cadena_travis)
        hits = tools._cache_it.cache_info().hits
        self.assertFalse(hits, 'Cache was not cleared')
        tools.cache_it(self.cadena_travis)
        hits = tools._cache_it.cache_info().hits
        self.assertTrue(hits == 1, 'Cache hits not increased 1 after cleared')

    def test_008_s3(self):
        """Cache on amazon is working properly"""
        if not environ.get('TRAVIS') == 'true':
            # This set of tests can be run just locally with properly set
            # amazon credentials, this will synchronize the xsd files
            # (and others) it does not make sense in travis.
            url = self.test_url_plain
            url_on_s3 = self.s3_url_plain
            new_url = tools.cache_s3(url, self.test_plain)
            self.assertEqual(new_url, url_on_s3,
                             'The url on amazon was not setted properly '
                             'got %s' % new_url)
            new_url = tools.cache_s3(url_on_s3, self.test_plain)
            self.assertEqual(new_url, url_on_s3,
                             'The url on amazon wired not return what I expected'
                             ' properly got %s' % new_url)

            check_s3 = tools.check_s3('NOTEXISTSORNOTPROPERACL', 'url/no/exists')
            self.assertFalse(check_s3, 'checking a non existing bucket fails')
            check_s3 = tools.check_s3('s3.vauxoo.com', 'url/no/exists')
            self.assertFalse(check_s3, 'checking a non existing element fails')

    def test_008_force_s3_creation(self):
        """Updating XSD of CFDIv32 Only local ignored in travis"""
        # Moved to script in root folder.
        pass

    def test_009_get_payroll(self):
        """With a given valid dict an
        payroll object is created in debug_mode"""
        payroll = cfdv33.get_payroll(self.dict_payroll, debug_mode=True)
        self.assertTrue(payroll.document,
                        'A valid dictionary gave error with the payroll %s' % payroll.ups)
        self.assertFalse(
            bool(payroll.ups),
            'A valid dictionary gave error payroll %s' % payroll.document)

    def test_010_get_cfdi33(self):
        """With a given valid dict an
        cfdi33 object is created in debug_mode"""
        invoice = cfdv33.get_cfdi(self.dict_cfdi33, debug_mode=True)
        self.assertTrue(
            invoice.document,
            'A valid dictionary gave error with the CFDI 3.3 %s' % invoice.ups)
        self.assertFalse(
            bool(invoice.ups),
            'A valid dictionary gave error CFDI 3.3 %s' % invoice.document)

    def test_014_get_errored_cfdi33(self):
        """With a given invalid dict raise properly errors on ups object"""
        invoice = cfdv33.get_cfdi(self.dict_invoice_basic_33_errored)
        self.assertTrue(bool(invoice.ups),
                        'An invalid dictionary gave a '
                        'valid output, that is wrong.')
        # Ok it failed!, then we assert if
        # the message is the one I expected for.
        self.assertTrue(invoice.ups.message.find('Emisor') > 0,
                        'The expected failed entry Emisor was erroneous. %s' % invoice.ups.message)

        invoice = cfdv33.get_cfdi({})
        self.assertTrue(bool(invoice.ups),
                        'An empty dict should give me the validation')

        invoice = cfdv33.get_cfdi(self.dict_invoice_basic_33_false)
        self.assertNotIn('False', invoice.ups.message,
                         'Passing a False value return a False string which is'
                         'incorrect  %s ' % invoice.ups.message)

    def test_015_get_payment10(self):
        """With a given valid dict an
        payment object is created in debug_mode"""
        payment = cfdv33.get_payment10(self.dict_payment, debug_mode=True)
        self.assertTrue(payment.document,
                        'A valid dictionary gave error with the payment %s' % payment.ups)
        self.assertFalse(
            bool(payment.ups),
            'A valid dictionary gave error payroll %s' % payment.document)


if __name__ == '__main__':
    import sys
    sys.exit(unittest.main())



#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""
test_signatory
----------------------------------

Tests for `signatory` module.
"""

import unittest
from cfdilib.signatory import Signatory
import suds


class TestSignatory(unittest.TestCase):

    def setUp(self):
        pass

    def tearDown(self):
        pass

    def test_001_managing_known_errors(self):
        """Given a malformed url"""
        signatory = Signatory('test', 'test_usr', 'pass_test')
        signatory.sign('<a>XML</a>')
        self.assertTrue(
            signatory.message.find('unknown') >= 0,
            'Passing an incorrect url the message was not the expected')

    def test_002_managing_known_errors(self):
        """Given a well formed url but unkown host"""
        url = 'http://url.do.not.exist.com'
        signatory = Signatory(url, 'test_usr', 'pass_test')
        signatory.sign('<a>XML</a>')
        self.assertTrue(signatory.message.find(url) >= 0,
                        'Passing an incorrect url the message '
                        'was not the expected: %s' % signatory.message)

    def test_003_managing_known_errors(self):
        """Given a well formed url but incorrect user password
        client is setted correctly."""
        # TODO: Mock this test
        url = 'http://demo-facturacion.finkok.com/servicios/soap/stamp.wsdl'
        signatory = Signatory(url, 'xx', 'yy')
        signatory.sign('<a>XML</a>')
        self.assertTrue(isinstance(signatory.client, suds.client.Client),
                        'Passing a correct  url the '
                        'client object is not what I expected')
        self.assertTrue(signatory.message is None,
                        'The message on the invoice object should'
                        ' be False and it is not')



# -*- coding: utf-8 -*-

from .cfdilib import BaseDocument
from .tools import tools


class CFDv33(BaseDocument):
    """Invoice document to comply with
    cfdi: v3.3 for Invoice Mexico Standards."""

    def __init__(self, dict_invoice, debug_mode=False):
        self.template_fname = 'cfdv33.xml'
        # We explicitly cached into s3 with the local test then ensure use
        # the s3 url to use our cache. remove the tools.s3_url if you want to
        # load first in the future
        self.xslt_fname = tools.s3_url(
            'http://s3.vauxoo.com/sitio_internet/cfd/3/cadenaoriginal_3_2/cadenaoriginal_3_2.xslt')
        self.global_namespace = 'http://www.sat.gob.mx/sitio_internet/cfd'
        self.set_template(self.template_fname)
        super(CFDv33, self).__init__(dict_invoice, debug_mode=debug_mode)
        # This method must be called from the inherited
        #   __init__ always AFTER the super.

    def set_template(self, template_fname):
        self.template = super(CFDv33, self).set_template(template_fname)

    def set_schema(self, schema_fname):
        self.schema = super(CFDv33, self).set_schema(schema_fname)

    def set_xslt(self):
        # TODO: Standarize the schema in this way also,
        #       we can not use different algorithms here
        self.xstl = super(CFDv33, self).set_xslt()


def get_cfdi(dict_invoice, debug_mode=False):
    return CFDv33(dict_invoice, debug_mode=debug_mode)


class Payroll12(BaseDocument):
    """Invoice document to comply with
    cfdi: v3.3 for Invoice Mexico Standards."""

    def __init__(self, dict_invoice, debug_mode=False):
        self.template_fname = 'payroll12.xml'
        # We explicitly cached into s3 with the local test then ensure use
        # the s3 url to use our cache. remove the tools.s3_url if you want to
        # load first in the future
        # TODO - Change by reference to payroll v1.2
        self.xslt_fname = tools.s3_url(
            'http://s3.vauxoo.com/sitio_internet/cfd/3/cadenaoriginal_3_2/cadenaoriginal_3_2.xslt')
        self.global_namespace = 'http://www.sat.gob.mx/sitio_internet/cfd'
        self.set_template(self.template_fname)
        super(Payroll12, self).__init__(dict_invoice, debug_mode=debug_mode)
        # This method must be called from the inherited
        #   __init__ always AFTER the super.

    def set_template(self, template_fname):
        self.template = super(Payroll12, self).set_template(template_fname)

    def set_schema(self, schema_fname):
        schema_fname = 'cfdv33.xsd'
        self.schema = super(Payroll12, self).set_schema(schema_fname)

    def set_xslt(self):
        # TODO: Standarize the schema in this way also,
        #       we can not use different algorithms here
        self.xstl = super(Payroll12, self).set_xslt()


def get_payroll(dict_payroll, debug_mode=False):
    return Payroll12(dict_payroll, debug_mode=debug_mode)


class Payment10(BaseDocument):
    """Invoice document to comply with
    cfdi: v3.3 for Invoice Mexico Standards. Complement to payments version 1.0
    """

    def __init__(self, dict_invoice, debug_mode=False):
        self.template_fname = 'payment10.xml'
        # We explicitly cached into s3 with the local test then ensure use
        # the s3 url to use our cache. remove the tools.s3_url if you want to
        # load first in the future
        # TODO - Change by reference to payroll v1.2
        self.xslt_fname = tools.s3_url(
            'http://s3.vauxoo.com/sitio_internet/cfd/3/cadenaoriginal_3_2/cadenaoriginal_3_2.xslt')
        self.global_namespace = 'http://www.sat.gob.mx/sitio_internet/cfd'
        self.set_template(self.template_fname)
        super(Payment10, self).__init__(dict_invoice, debug_mode=debug_mode)
        # This method must be called from the inherited
        #   __init__ always AFTER the super.

    def set_template(self, template_fname):
        self.template = super(Payment10, self).set_template(template_fname)

    def set_schema(self, schema_fname):
        schema_fname = 'cfdv33.xsd'
        self.schema = super(Payment10, self).set_schema(schema_fname)

    def set_xslt(self):
        # TODO: Standarize the schema in this way also,
        #       we can not use different algorithms here
        self.xstl = super(Payment10, self).set_xslt()


def get_payment10(dict_payment, debug_mode=False):
    return Payment10(dict_payment, debug_mode=debug_mode)



#!/usr/bin/env python
# -*- coding: utf-8 -*-


try:
    from setuptools import setup
except ImportError:
    from distutils.core import setup


with open('README.rst') as readme_file:
    readme = readme_file.read()

with open('HISTORY.rst') as history_file:
    history = history_file.read()

requirements = [
    "boto3",
    "click",
    "jinja2",
    # "lxml",  # Even if we need this in ubuntu you need use the system site
    #  package.
    "backports.functools_lru_cache",
]

test_requirements = [
    # TODO: put package test requirements here
]

setup(
    name='cfdilib',
    version='0.7.4',
    description="Library to manage xml CDFI from " +
    "python and sign with several pacs.",
    long_description=readme + '\n\n' + history,
    author="Vauxoo OpenSource Specialists.",
    author_email='mexico@vauxoo.com',
    url='https://github.com/vauxoo/cfdilib',
    packages=[
        'cfdilib',
    ],
    package_dir={'cfdilib':
                 'cfdilib'},
    include_package_data=True,
    install_requires=requirements,
    license="ISCL",
    zip_safe=False,
    keywords='cfdilib',
    classifiers=[
        'Development Status :: 2 - Pre-Alpha',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: ISC License (ISCL)',
        'Natural Language :: English',
        "Programming Language :: Python :: 2",
        'Programming Language :: Python :: 2.6',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.3',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
    ],
    test_suite='tests',
    tests_require=test_requirements,
    entry_points='''
        [console_scripts]
        cfdicli=cfdilib.cfdicli:cli
    ''',
)



# -*- coding: utf-8 -*-

from .cfdilib import BaseDocument
from .tools import tools


class CFDv32(BaseDocument):
    """Invoice document to comply with
    cfdi: v3.2 for Invoice Mexico Standards."""

    def __init__(self, dict_invoice, debug_mode=False):
        self.template_fname = 'cfdv32.xml'
        # We explicitly cached into s3 with the local test then ensure use
        # the s3 url to use our cache. remove the tools.s3_url if you want to
        # load first in the future
        self.xslt_fname = \
            tools.s3_url('http://s3.vauxoo.com/sitio_internet/cfd/3/cadenaoriginal_3_2/cadenaoriginal_3_2.xslt')
        self.global_namespace = 'http://www.sat.gob.mx/sitio_internet/cfd'
        self.set_template(self.template_fname)
        super(CFDv32, self).__init__(dict_invoice, debug_mode=debug_mode)
        # This method must be called from the inherited
        #   __init__ always AFTER the super.

    def set_template(self, template_fname):
        self.template = super(CFDv32, self).set_template(template_fname)

    def set_schema(self, schema_fname):
        self.schema = super(CFDv32, self).set_schema(schema_fname)

    def set_xslt(self):
        # TODO: Standarize the schema in this way also,
        #       we can not use different algorithms here
        self.xstl = super(CFDv32, self).set_xslt()


def get_invoice(dict_invoice, debug_mode=False):
    return CFDv32(dict_invoice, debug_mode=debug_mode)


class CFDICoA(BaseDocument):
    """CoA document
    cfdi: v1.3 for Accounting."""

    def __init__(self, dict_accounts, debug_mode=False):
        self.template_fname = 'cfdi13coa.xml'
        # We explicitly cached into s3 with the local test then ensure use
        # the s3 url to use our cache. remove the tools.s3_url if you want to
        # load first in the future
        self.xslt_fname = 'http://www.sat.gob.mx/esquemas/ContabilidadE/1_3/CatalogoCuentas/CatalogoCuentas_1_2.xslt'
        self.global_namespace = 'http://www.sat.gob.mx/esquemas/ContabilidadE/1_3/CatalogosParaEsqContE'
        self.set_template(self.template_fname)
        super(CFDICoA, self).__init__(dict_accounts, debug_mode=debug_mode)

    def set_template(self, template_fname):
        self.template = super(CFDICoA, self).set_template(template_fname)

    def set_schema(self, schema_fname):
        self.schema = super(CFDICoA, self).set_schema(schema_fname)

    def set_xslt(self):
        # TODO: Standarize the schema in this way also,
        #       we can not use different algorithms here
        self.xstl = super(CFDICoA, self).set_xslt()


def get_coa(dict_accounts, debug_mode=False):
    return CFDICoA(dict_accounts, debug_mode=debug_mode)

class CFDIBalance(BaseDocument):
    """Balance XML document
    cfdi: v1.3 for Accounting."""

    def __init__(self, dict_accounts, debug_mode=False):
        self.template_fname = 'cfdi13balance.xml'
        # We explicitly cached into s3 with the local test then ensure use
        # the s3 url to use our cache. remove the tools.s3_url if you want to
        # load first in the future
        self.xslt_fname = 'http://www.sat.gob.mx/esquemas/ContabilidadE/1_3/BalanzaComprobacion/BalanzaComprobacion_1_2.xslt'
        self.global_namespace = 'http://www.sat.gob.mx/esquemas/ContabilidadE/1_3/BalanzaComprobacion'
        self.set_template(self.template_fname)
        super(CFDIBalance, self).__init__(dict_accounts, debug_mode=debug_mode)

    def set_template(self, template_fname):
        self.template = super(CFDIBalance, self).set_template(template_fname)

    def set_schema(self, schema_fname):
        self.schema = super(CFDIBalance, self).set_schema(schema_fname)

    def set_xslt(self):
        # TODO: Standarize the schema in this way also,
        #       we can not use different algorithms here
        self.xstl = super(CFDIBalance, self).set_xslt()


def get_balance(dict_balance, debug_mode=False):
    return CFDIBalance(dict_balance, debug_mode=debug_mode)

class CFDIMoves(BaseDocument):
    """Balance XML document
    cfdi: v1.3 for Accounting."""

    def __init__(self, dict_accounts, debug_mode=False):
        self.template_fname = 'cfdi13moves.xml'
        # We explicitly cached into s3 with the local test then ensure use
        # the s3 url to use our cache. remove the tools.s3_url if you want to
        # load first in the future
        self.xslt_fname = 'http://www.sat.gob.mx/esquemas/ContabilidadE/1_3/PolizasPeriodo/PolizasPeriodo_1_2.xslt'
        self.global_namespace = 'http://www.sat.gob.mx/esquemas/ContabilidadE/1_3/CatalogosParaEsqContE'
        self.set_template(self.template_fname)
        super(CFDIMoves, self).__init__(dict_accounts, debug_mode=debug_mode)

    def set_template(self, template_fname):
        self.template = super(CFDIMoves, self).set_template(template_fname)

    def set_schema(self, schema_fname):
        self.schema = super(CFDIMoves, self).set_schema(schema_fname)

    def set_xslt(self):
        # TODO: Standarize the schema in this way also,
        #       we can not use different algorithms here
        self.xstl = super(CFDIMoves, self).set_xslt()


def get_moves(dict_moves, debug_mode=False):
    return CFDIMoves(dict_moves, debug_mode=debug_mode)
